#include "hal_file_transfer.h"
#include "tal_memory.h"
#include "tal_system.h"
#include "tal_poll_manager.h"


#define TUYA_SKD_DBG    1
#if defined (TUYA_SKD_DBG)
#include "tkl_uart.h"
#include "tlprintf.h"
#define hal_transfer_debug(...)   tkl_dbg_printf(__VA_ARGS__)
#else
#define hal_transfer_debug(...)
#endif

#define APP_TRY_SEND_TIME_MAX           5
#define IR_REC_TIMEOUT_MAX              (5*1000)/*红外发送5秒内发送未结束则任务发送失败，则进行异常处理*/
#define IR_SEND_TIMEOUT_MAX             (10*1000)/*红外发送5秒内发送未结束则任务发送失败，则进行异常处理*/

/********************************************

    client                           server

    <---------------------start notification
    notify respond ------------------------->

    block request -------------------------->
    <---------------------------block respond
                        .
                        .
                        .
    end request -------------------------->
    <---------------------------end respond

 *******************************************/

typedef struct {
    UINT8_T dest_ep;
    UINT8_T src_ep;
    UINT16_T dest_addr;
    UINT32_T random_time;
    ZG_ZCL_DATA_DIRECTION_E dir;
    ZG_CLUSTER_ID_E cluster_id;
    UINT8_T command_id;
    UINT8_T input_len;
    UINT8_T *input_buf;
    TAL_SEND_QOS_E qos;
}private_cmd_data_t;

static muliple_packet_config_t default_config;
static UINT32_T g_quick_poll_time = 0;

void multiple_packet_transfer_evt_handle(TIMER_ID timer_id, VOID_T *arg);
void send_end_request_deal(END_RESPOND_STATUS_T end_status,BOOL_T force);
static void send_block_respond(UINT16_T src_addr,block_respond_pdu_t pdu_buf);
void set_rec_wait_time(BOOL_T rec_enable);

extern void emberAfPluginEndDeviceSupportPollingNetworkEventHandler(void);
extern BOOL_T check_start_notification_status(UINT16_T cluster_id,UINT8_T cmd_type,UINT8_T cmd,UINT32_T total_size);

static UINT8_T g_app_try_send_time = APP_TRY_SEND_TIME_MAX;
static UINT16_T g_app_try_send_buf[APP_TRY_SEND_TIME_MAX] = {1000,2000,IR_REC_TIMEOUT_MAX,IR_REC_TIMEOUT_MAX,IR_REC_TIMEOUT_MAX};

static TIMER_ID sw_fast_poll_use = 0;
static TIMER_ID sw_rec_process_timeout_use = 0;
static TIMER_ID sw_send_process_timeout_use = 0;


/*************************************************************************************************
 * 
 * 
 *                                    分包工具部分
 * 
 * 
 * **************************************************************************************************/
void fresh_send_try_times(void)
{
    g_app_try_send_time = 0;
}

void end_send_try_times(void)
{
    g_app_try_send_time = APP_TRY_SEND_TIME_MAX;
    set_rec_wait_time(FALSE); 
}


void ext_set_quick_time(BOOL_T enable,UINT32_T slice_time)
{
    if(TRUE == enable){
        g_quick_poll_time = slice_time;
        tal_sw_timer_start(sw_fast_poll_use,g_quick_poll_time,TAL_TIMER_ONCE);
    }
    else{
        if(TRUE==tal_sw_timer_is_running(sw_fast_poll_use)){
            tal_sw_timer_stop(sw_fast_poll_use);                     
        }
    }
}

void set_rec_wait_time(BOOL_T rec_enable)
{
    if(TRUE == rec_enable){
        fresh_send_try_times();
        tal_sw_timer_start(sw_rec_process_timeout_use,g_app_try_send_buf[g_app_try_send_time],TAL_TIMER_ONCE);
    }
    else{
        if(TRUE==tal_sw_timer_is_running(sw_rec_process_timeout_use)){
            tal_sw_timer_stop(sw_rec_process_timeout_use);
        }
        g_app_try_send_time = APP_TRY_SEND_TIME_MAX;
        if(FALSE==tal_sw_timer_is_running(sw_send_process_timeout_use)){
            ext_set_quick_time(FALSE,0);                 
        }
    }
}

void set_send_wait_time(BOOL_T send_enable)
{
    if(TRUE == send_enable){
        tal_sw_timer_start(sw_send_process_timeout_use,IR_SEND_TIMEOUT_MAX,TAL_TIMER_ONCE);
    }
    else{        
        if(TRUE==tal_sw_timer_is_running(sw_send_process_timeout_use)){
            tal_sw_timer_stop(sw_send_process_timeout_use);                
        }
        if(FALSE==tal_sw_timer_is_running(sw_rec_process_timeout_use)){
            ext_set_quick_time(FALSE,0);                 
        }
    }
}

UINT8_T u8_checksum(UINT8_T*data,UINT8_T length)
{
    UINT8_T return_value = 0;
    UINT8_T i = 0;
    for(i=0;i<length;i++){
        return_value += data[i];
    }
    return (UINT8_T)return_value;
}

UINT8_T get_frist_index_ep(void)
{
    return 1;
}


BOOL_T check_other_file_continue(void)
{
    return FALSE;
}

/*********************************************************************************
 * Function:
 * Description:    函数发送结束之后的结果回调函数
 * Input:
 * Outpot:
 * Return:
 * History:
 *******************************************************************************/
void hal_send_data_result_callback(TAL_SEND_ST_E result, TAL_ZG_SEND_DATA_T * data)
{
    if(default_config.send_result){
        default_config.send_result(result,data);
    }
}

/*********************************************************************************
 * Function:
 * Description:   report cmd data 
 * Input:
 * Outpot:
 * Return:
 * History:
 *******************************************************************************/
static void dev_send_cluster_cmd_data(private_cmd_data_t input)
{     
    TAL_ZG_SEND_DATA_T ret_data;
    tal_system_memset(&ret_data, 0, sizeof(TAL_ZG_SEND_DATA_T));
    ret_data.random_time = (UINT16_T)tal_system_get_random(input.random_time);
    ret_data.command_id = input.command_id;
    ret_data.frame_type = ZG_ZCL_FRAME_TYPE_SPEC_TO_CLUSTER;
    ret_data.direction = input.dir;
    ret_data.qos = input.qos;
    ret_data.addr.mode = SEND_MODE_DEV;
    ret_data.addr.type.dev.src_ep = input.src_ep;
    ret_data.addr.type.dev.dst_ep = input.dest_ep;
    ret_data.addr.type.dev.cluster_id = input.cluster_id;
    ret_data.addr.type.dev.dst_addr = input.dest_addr;
    ret_data.data.private.len = input.input_len;
    tal_system_memcpy(ret_data.data.private.data,input.input_buf,input.input_len);
    tal_zg_send_data(&ret_data, hal_send_data_result_callback, 1000);
}



/*************************************************************************************************
 * 
 * 
 *                                    分包接收部分
 * 
 * 
 * **************************************************************************************************/

///< client to Server
/**
 * @description: client 发送接收start notify后的notify respond
 * @param {*data} PDU data,{len} PDU len
 * @return: none
 */
void transfer_send_start_respond(UINT16_T dest_addr,UINT8_T dest_ep,UINT8_T src_ep,start_respond_pdu_t pdu_buf)
{
    private_cmd_data_t cmd_buf;
    cmd_buf.src_ep = src_ep;
    cmd_buf.dest_ep = dest_ep;
    cmd_buf.dest_addr = dest_addr;
    cmd_buf.random_time = 0;
    cmd_buf.dir = ZG_ZCL_DATA_CLIENT_TO_SERVER;
    cmd_buf.cluster_id = CLUSTER_MULTIPLE_PACKET_TRANSFER_CLUSTER_ID;
    cmd_buf.command_id = CMD_TRANSFER_START_RESPOND_COMMAND_ID;
    cmd_buf.input_len = sizeof(start_respond_pdu_t);
    cmd_buf.input_buf = (UINT8_T*)&pdu_buf;
    cmd_buf.qos = QOS_1;
    dev_send_cluster_cmd_data(cmd_buf);
}


///< Server to client
/**
 * @description: client send,block request
 * @param {*data} PDU data,{len} PDU len
 * @return: none
 */
void transfer_send_block_request_send(UINT16_T dest_addr,UINT8_T dest_ep,UINT8_T src_ep,block_request_pdu_t pdu_buf)
{
    private_cmd_data_t cmd_buf;
    cmd_buf.src_ep = src_ep;
    cmd_buf.dest_ep = dest_ep;
    cmd_buf.dest_addr = dest_addr;
    cmd_buf.random_time = 0;//run_info.rec.total_info.random_time;
    cmd_buf.dir = ZG_ZCL_DATA_CLIENT_TO_SERVER;
    cmd_buf.cluster_id = CLUSTER_MULTIPLE_PACKET_TRANSFER_CLUSTER_ID;
    cmd_buf.command_id = CMD_TRANSFER_BLOCK_REQUEST_COMMAND_ID;
    cmd_buf.input_len = sizeof(block_request_pdu_t);
    cmd_buf.input_buf = (UINT8_T*)&pdu_buf;
    cmd_buf.qos = QOS_0;
    dev_send_cluster_cmd_data(cmd_buf);
}


///< client to Server
/**
 * @description: client 主动发送end request
 * @param {*data} PDU data,{len}PDU len
 * @return: none
 */
void transfer_send_end_request(UINT16_T dest_addr,UINT8_T dest_ep,UINT8_T src_ep,end_request_pdu_t pdu_buf)
{
    hal_transfer_debug("send end request \r\n");
    private_cmd_data_t cmd_buf;
    cmd_buf.src_ep = src_ep;
    cmd_buf.dest_ep = dest_ep;
    cmd_buf.dest_addr = dest_addr;
    cmd_buf.random_time = 0;
    cmd_buf.dir = ZG_ZCL_DATA_CLIENT_TO_SERVER;
    cmd_buf.cluster_id = CLUSTER_MULTIPLE_PACKET_TRANSFER_CLUSTER_ID;
    cmd_buf.command_id = CMD_TRANSFER_END_REQUEST_COMMAND_ID;
    cmd_buf.input_len = sizeof(end_request_pdu_t);
    cmd_buf.input_buf = (UINT8_T*)&pdu_buf;
    cmd_buf.qos = QOS_1;
    dev_send_cluster_cmd_data(cmd_buf);
}


///< Server to client
/**
 * @description: start notify格式校验
 * @param {*data} PDU data,{len}PDU len
 * @return: none
 */
static BOOL_T start_notification_format_check(UINT8_T *data,UINT8_T len)
{
    if((sizeof(start_notification_pdu_t) != len) || (NULL == data)){
        return FALSE;
    }
    return TRUE;
} 


///< Server to client
/**
 * @description: block respond格式校验
 * @param {*data} PDU data,{len}PDU len
 * @return: none
 */
static BOOL_T block_respond_format_check(UINT8_T *data,UINT8_T len)
{
    if((NULL == data) || (offsetof(block_respond_pdu_t,length)>=len)){
        return FALSE;
    }
    else{
        if((sizeof(block_respond_pdu_t)-sizeof(UINT8_T*)+data[offsetof(block_respond_pdu_t,length)])!= len){
            return FALSE;
        }        
    }
    return TRUE;
}


///< Server to client
/**
 * @description: end respond格式校验
 * @param {*data} PDU data,{len}PDU len
 * @return: none
 */
static BOOL_T rec_end_respond_format_check(UINT8_T *data,UINT8_T len)
{
    if((sizeof(end_respond_pdu_t) != len) || (NULL == data)){
        return FALSE;
    }
    return TRUE;
}


///< Server to client
/**
 * @description: start notify 处理部分
 * @param {*data} PDU data,{len}PDU len
 * @return: none
 */
BOOL_T rec_start_notification_deal(UINT16_T dest_addr,UINT8_T dest_ep,UINT8_T src_ep,TAL_ZCL_MSG_T frame)
{
    BOOL_T notify_result = FALSE;
    start_respond_pdu_t *pdu_buf = (start_respond_pdu_t*)tal_malloc(sizeof(start_respond_pdu_t));
    REC_MULIPLE_PACKET_ERROR_CODE_T result = REC_FAIL_OTHER;
    BOOL_T send_block_req_flag = FALSE;
    
    if(pdu_buf){
        tal_system_memset(pdu_buf,0x00,sizeof(start_respond_pdu_t));
        if(start_notification_format_check(frame.payload,frame.length))
        {            
            tal_system_memcpy((UINT8_T*)&(pdu_buf->file_id),(UINT8_T*)(frame.payload),frame.length);
            if(default_config.rec_notify_req)
            {
                if(default_config.rec_notify_req){
                    if(!check_start_notification_status(pdu_buf->cluster_id,pdu_buf->cmd_type,pdu_buf->cmd,pdu_buf->total_size)){
                        result = REC_FAIL_OTHER;
                        pdu_buf->notify_status = NOTIFY_RES_OTHER_ERROR;
                    }
                    else{
                        result = default_config.rec_notify_req(dest_addr,*(start_notification_pdu_t*)(frame.payload));                
                        if(REC_SUCCESS_START == result)
                        {   
                            pdu_buf->notify_status = NOTIFY_RES_SUCCESS;   
                            notify_result = TRUE;          
                        }
                        else{
                            pdu_buf->notify_status = NOTIFY_RES_BUSY_ERROR;
                        }
                    }
                }
            }
            else{
                pdu_buf->notify_status = NOTIFY_RES_OTHER_ERROR;
            } 
            if(NOTIFY_RES_SUCCESS == pdu_buf->notify_status){            
                send_block_req_flag = TRUE;                 
            }
        }
        else{
            tal_free(pdu_buf);
            return FALSE;/*格式不正确则避免内存操作溢出等，采用不回复*/
        }
        if(result != REC_NO_RESPOND){/*相同的file 及 addr 对应的传输进程，不再进行回复*/
            transfer_send_start_respond(dest_addr,0x01,src_ep,*pdu_buf);            
        }
        if(send_block_req_flag){
            fresh_send_try_times();
            send_block_request_deal();
        }
        tal_free(pdu_buf);
    }
    else{
        hal_transfer_debug("tal_malloc fail\r\n");
    }
    return notify_result;
}



///< client to Server
/**
 * @description: client 主动发送end request
 * @param {*data} PDU data,{len}PDU len
 * @return: none
 */
void send_end_request_deal(END_RESPOND_STATUS_T end_status,BOOL_T force)
{
    end_request_pdu_t* pdu_buf = (end_request_pdu_t*)tal_malloc(sizeof(end_request_pdu_t));
    UINT16_T src_addr=0;

    if(pdu_buf){
        tal_system_memset((UINT8_T*)pdu_buf,0x00,sizeof(end_request_pdu_t));
        if(default_config.send_end_req){
            if(default_config.send_end_req(&src_addr,pdu_buf,end_status,force)){
                transfer_send_end_request(src_addr,0x01,get_frist_index_ep(),*pdu_buf);                
            }      
        }
        tal_free(pdu_buf);
    }
    else{
        hal_transfer_debug("tal_malloc fail\r\n");
    }
}


///< client to Server
/**
 * @description: client 主动发送block request
 * @param {*data} PDU data,{len}PDU len
 * @return: none
 */
void send_block_request_deal(void)
{
    block_request_pdu_t* pdu_buf = (block_request_pdu_t*)tal_malloc(sizeof(block_request_pdu_t));
    UINT16_T src_addr=0;

    if(pdu_buf){
        tal_system_memset((UINT8_T*)pdu_buf,0x00,sizeof(block_request_pdu_t));
        if(default_config.send_block_req){
            if(default_config.send_block_req(&src_addr,pdu_buf)){
                transfer_send_block_request_send(src_addr,0x01,get_frist_index_ep(),*pdu_buf);                
            }     
        }
        tal_free(pdu_buf);
    }
    else{
        hal_transfer_debug("tal_malloc fail\r\n");
    }
}


UINT32_T get_pdu_file_offset(UINT8_T* pdu_buf,UINT8_T offset)
{
    if(pdu_buf){
        hal_transfer_debug("file offset: %x %x %x %x\r\n",*((UINT8_T*)pdu_buf+offset),*((UINT8_T*)pdu_buf+offset+1),*((UINT8_T*)pdu_buf+offset+2),*((UINT8_T*)pdu_buf+offset+3));
        return (UINT32_T)pdu_buf[offset] | (UINT32_T)pdu_buf[offset+1]<<8 | (UINT32_T)pdu_buf[offset+2]<<16 | (UINT32_T)pdu_buf[offset+3]<<24;
    }
}

///< Server to client
/**
 * @description: client 接收处理block request后server发送的block respond
 * @param {*data} PDU data,{len}PDU len
 * @return: none
 */
BOOL_T rec_block_respond_deal(UINT16_T dest_addr,UINT8_T dest_ep,UINT8_T src_ep,TAL_ZCL_MSG_T frame)
{
    BOOL_T block_res_result = FALSE;
    block_respond_pdu_t *pdu_buf = (block_respond_pdu_t*)tal_malloc(sizeof(block_respond_pdu_t));
    BOOL_T check_result = FALSE;
    REC_MULIPLE_PACKET_ERROR_CODE_T result = REC_FAIL_OTHER;
    UINT8_T temp_sum = 0; 

    if(pdu_buf){    
        tal_system_memset(pdu_buf,0x00,sizeof(block_respond_pdu_t));
        if(block_respond_format_check(frame.payload,frame.length)){            
            tal_system_memcpy((UINT8_T*)pdu_buf,(UINT8_T*)(frame.payload),BLOCK_RESPOND_HEAD_LEN);
            pdu_buf->data = (UINT8_T*)frame.payload+BLOCK_RESPOND_HEAD_LEN;
            pdu_buf->checksum = ((UINT8_T*)frame.payload)[frame.length-1];
            if(BLOCK_RES_SUCCESS == pdu_buf->block_status){
                if((frame.payload) && (frame.length>1)){
                    temp_sum = u8_checksum((UINT8_T*)frame.payload+BLOCK_RESPOND_HEAD_LEN,(frame.length-BLOCK_RESPOND_HEAD_LEN-1));
                    hal_transfer_debug("file offset: %d \r\n",pdu_buf->file_offset);
                    if(pdu_buf->checksum == temp_sum){/*进行应用层数据帧一帧校验*/  
                        check_result = TRUE;
                    }
                    else{
                        check_result = FALSE;
                    }  
                    if((default_config.rec_block_res) && (TRUE == check_result)){
                        result = default_config.rec_block_res(dest_addr,pdu_buf->file_id,check_result,get_pdu_file_offset(pdu_buf,offsetof(block_respond_pdu_t,file_offset)),\
                        pdu_buf->data,*(UINT8_T*)((UINT8_T*)pdu_buf+offsetof(block_respond_pdu_t,length)));  
                        if(REC_SUCCESS_CONTINUE==result){
                            block_res_result=TRUE;
                        }
                    }
                    /*TODO，如果server返回的一帧数据offset不对怎么处理，暂定不进行回复*/ 
                }
            }
        }
        else{
            tal_free(pdu_buf);
            return  FALSE;/*格式不正确则避免内存操作溢出等，采用不回复*/
        }   
        if(REC_SUCCESS_END == result){
            send_end_request_deal(END_RES_SUCCESS,FALSE);
        }
        else if(REC_SUCCESS_CONTINUE == result){
            fresh_send_try_times();
            send_block_request_deal();
        }
        tal_free(pdu_buf);
    }
    else{
        hal_transfer_debug("tal_malloc fail\r\n");
    }
    return block_res_result;
}



///< Server to client
/**
 * @description: client 接收处理发送end request后server发送的end respond
 * @param {*data} PDU data,{len}PDU len
 * @return: none
 */
BOOL_T rec_rec_end_respond_deal(UINT16_T dest_addr,UINT8_T dest_ep,UINT8_T src_ep,TAL_ZCL_MSG_T frame)
{
    BOOL_T end_res_result = FALSE;
    end_respond_pdu_t *pdu_buf = (end_respond_pdu_t*)tal_malloc(sizeof(end_respond_pdu_t));

    if(pdu_buf){
        if(rec_end_respond_format_check(frame.payload,frame.length)){
            tal_system_memset((UINT8_T*)pdu_buf,0x00,END_RESPOND_LEN);
            tal_system_memcpy((UINT8_T*)pdu_buf,(UINT8_T*)(frame.payload),END_RESPOND_LEN);
            if(default_config.rec_end_res){
        #if 0
                default_config.rec_end_res(dest_addr,pdu_buf->file_id);/*状态刷新为idle应该由msg_rec函数处理结束时进行，否则会造成接收数据被错误修改*/   
        #endif  
                end_res_result = TRUE;       
            }  
        } 
        tal_free(pdu_buf);      
    }
    else{
        hal_transfer_debug("tal_malloc fail\r\n");
    }
    return end_res_result;
}



/*************************************************************************************************
 * 
 * 
 *                                    分包发送部分
 * 
 * 
 * **************************************************************************************************/


///< Server to client
/**
 * @description:  server send,start notification 
 * @param {*data} PDU data,{len}PDU len
 * @return: none
 */
void transfer_start_notification_send(UINT16_T dest_addr,UINT8_T dest_ep,UINT8_T src_ep,start_notification_pdu_t pdu_buf)
{
    hal_transfer_debug("send start_notification \r\n");
    private_cmd_data_t cmd_buf;
    cmd_buf.src_ep = src_ep;
    cmd_buf.dest_ep = dest_ep;
    cmd_buf.dest_addr = dest_addr;
    cmd_buf.random_time = 0;
    cmd_buf.dir = ZG_ZCL_DATA_CLIENT_TO_SERVER;
    cmd_buf.cluster_id = CLUSTER_MULTIPLE_PACKET_TRANSFER_CLUSTER_ID;
    cmd_buf.command_id = CMD_TRANSFER_START_NOTIFICATION_COMMAND_ID;
    cmd_buf.input_len = sizeof(start_notification_pdu_t);
    cmd_buf.input_buf = (UINT8_T*)&pdu_buf;
    cmd_buf.qos = QOS_1;
    dev_send_cluster_cmd_data(cmd_buf);
} 



///< Server to client
/**
 * @description: server send,block responds
 * @param {*data} PDU data,{len}PDU len
 * @return: none
 */
void transfer_block_respond_send(UINT16_T dest_addr,UINT8_T dest_ep,UINT8_T src_ep,block_respond_pdu_t pdu_buf)
{
    hal_transfer_debug("send block_respond \r\n");
    private_cmd_data_t cmd_buf;
    UINT8_T send_len = sizeof(block_respond_pdu_t)-sizeof(UINT8_T*)+pdu_buf.length;

    if(send_len>DEF_BLOCK_REQ_MAX_SIZE){
        return;
    }
    cmd_buf.src_ep = src_ep;
    cmd_buf.dest_ep = dest_ep;
    cmd_buf.dest_addr = dest_addr;
    cmd_buf.random_time = 0;
    cmd_buf.dir = ZG_ZCL_DATA_SERVER_TO_CLIENT;
    cmd_buf.cluster_id = CLUSTER_MULTIPLE_PACKET_TRANSFER_CLUSTER_ID;
    cmd_buf.command_id = CMD_TRANSFER_BLOCK_RESPOND_COMMAND_ID;
    cmd_buf.input_len = send_len;
    cmd_buf.input_buf = tal_malloc(cmd_buf.input_len);
    if(cmd_buf.input_buf){
        tal_system_memset(cmd_buf.input_buf,0x00,sizeof(cmd_buf.input_len));
        tal_system_memcpy(cmd_buf.input_buf,(UINT8_T*)&pdu_buf,offsetof(block_respond_pdu_t,data));
        tal_system_memcpy(cmd_buf.input_buf+offsetof(block_respond_pdu_t,data),\
                pdu_buf.data,send_len-offsetof(block_respond_pdu_t,data)-sizeof(UINT8_T));
        cmd_buf.input_buf[cmd_buf.input_len-1] = pdu_buf.checksum;    
        cmd_buf.qos = QOS_1;
        dev_send_cluster_cmd_data(cmd_buf); 
        tal_free(cmd_buf.input_buf); 
    }
    else{
        hal_transfer_debug("tal_malloc fail\r\n");
    }
}


///< Server to client
/**
 * @description: server send,end responds 
 * @param {*data} PDU data,{len}PDU len
 * @return: none
 */
void transfer_rec_end_respond_send(UINT16_T dest_addr,UINT8_T dest_ep,UINT8_T src_ep,end_respond_pdu_t pdu_buf)
{
    hal_transfer_debug("send end_respond \r\n");
    private_cmd_data_t cmd_buf;
    cmd_buf.src_ep = src_ep;
    cmd_buf.dest_ep = dest_ep;
    cmd_buf.dest_addr = dest_addr;
    cmd_buf.random_time = 0;
    cmd_buf.dir = ZG_ZCL_DATA_SERVER_TO_CLIENT;
    cmd_buf.cluster_id = CLUSTER_MULTIPLE_PACKET_TRANSFER_CLUSTER_ID;
    cmd_buf.command_id = CMD_TRANSFER_END_RESPOND_COMMAND_ID;
    cmd_buf.input_len = sizeof(end_respond_pdu_t);
    cmd_buf.input_buf = (UINT8_T*)&pdu_buf;
    cmd_buf.qos = QOS_1;
    dev_send_cluster_cmd_data(cmd_buf); 
}




///< client to Server
/**
 * @description: Server rec,start notify responds
 * @param {*data} PDU data,{len} PDU len
 * @return: none
 */
static BOOL_T rec_start_respond_format_check(UINT8_T *data,UINT8_T len)
{   
    if((sizeof(start_respond_pdu_t) != len) || (NULL == data)){
        return FALSE;
    }
    return TRUE;
}


///< client to Server
/**
 * @description: Server rec,block request
 * @param {*data} PDU data,{len} PDU len
 * @return: none
 */
static BOOL_T rec_block_request_format_check(UINT8_T *data,UINT8_T len)
{
    if((sizeof(block_request_pdu_t) != len) || (NULL == data)){
        return FALSE;
    }
    return TRUE;
}


///< client to Server
/**
 * @description: Server rec,end request
 * @param {*data} PDU data,{len}PDU len
 * @return: none
 */
static BOOL_T rec_end_request_format_check(UINT8_T *data,UINT8_T len)
{
    if((sizeof(end_request_pdu_t) != len) || (NULL == data)){
        return FALSE;
    }
    return TRUE;
} 


///< Server to client
/**
 * @description: 该函数为app层可以调用的发送数据起始start notify request函数，表示有一帧数据需要发送
 * @param {*data} PDU data,{len}PDU len
 * @return: none
 */
void send_start_notify_deal(UINT16_T src_addr,start_notification_pdu_t pdu_buf)
{
    transfer_start_notification_send(src_addr,0x01,get_frist_index_ep(),pdu_buf); 
    ext_set_quick_time(TRUE,QUICK_POLL_SLICE_TIME);    
    set_send_wait_time(TRUE);    
}



///< client to server
/**
 * @description: 接收到client发送的start respond后的处理函数
 * @param {*data} PDU data,{len}PDU len
 * @return: none
 */
BOOL_T rec_start_respond_deal(UINT16_T dest_addr,UINT8_T dest_ep,UINT8_T src_ep,TAL_ZCL_MSG_T frame)
{
    BOOL_T start_res_result=FALSE;
    start_respond_pdu_t *pdu_buf = (start_respond_pdu_t*)tal_malloc(sizeof(start_respond_pdu_t));

    if(pdu_buf){
        if(rec_start_respond_format_check(frame.payload,frame.length)){
            tal_system_memset((UINT8_T*)pdu_buf,0x00,sizeof(block_respond_pdu_t));          
            tal_system_memcpy((UINT8_T*)pdu_buf,(UINT8_T*)(frame.payload),sizeof(block_respond_pdu_t));
            if(NOTIFY_RES_SUCCESS == pdu_buf->notify_status){
                if(default_config.rec_notify_res){
                    default_config.rec_notify_res(dest_addr,*pdu_buf);
                    start_res_result=TRUE;
                }                
            }
        }
        tal_free(pdu_buf);         
    }
    else{
        hal_transfer_debug("tal_malloc fail\r\n");
    }
    return start_res_result;
}



///< client to server
/**
 * @description: 接收到client发送的block reqest后的处理函数
 * @param {*data} PDU data,{len}PDU len
 * @return: none
 */
BOOL_T rec_block_request_deal(UINT16_T dest_addr,UINT8_T dest_ep,UINT8_T src_ep,TAL_ZCL_MSG_T frame)
{
    BOOL_T block_req_result=FALSE;
    block_request_pdu_t *req_pdu_buf = (block_request_pdu_t*)tal_malloc(sizeof(block_request_pdu_t));
    block_respond_pdu_t *res_pdu_buf = (block_respond_pdu_t*)tal_malloc(sizeof(block_respond_pdu_t));

    if((req_pdu_buf)&&(res_pdu_buf))
    {
        if(rec_block_request_format_check(frame.payload,frame.length))
        {
            tal_system_memset((UINT8_T*)req_pdu_buf,0x00,sizeof(block_request_pdu_t));          
            tal_system_memcpy((UINT8_T*)req_pdu_buf,(UINT8_T*)(frame.payload),sizeof(block_request_pdu_t));
            if(default_config.rec_block_req)
            {
                if(SEND_BLOCK_SUCCESS == default_config.rec_block_req(dest_addr,*req_pdu_buf))
                {
                    if((default_config.send_block_res) && (res_pdu_buf))
                    {
                        if(TRUE == default_config.send_block_res(&dest_addr,res_pdu_buf))
                        {
                            send_block_respond(dest_addr,*res_pdu_buf);
                            block_req_result=TRUE;
                        }
                    }
                }
            }                
        }
    }
    else{
        hal_transfer_debug("tal_malloc fail\r\n");
    }
    if(res_pdu_buf){
        tal_free(res_pdu_buf);
    }
    if(req_pdu_buf){
        tal_free(req_pdu_buf);          
    }
    return block_req_result;
}


///< client to server
/**
 * @description: 接收到client发送的end request后的处理函数
 * @param {*data} PDU data,{len}PDU len
 * @return: none
 */
BOOL_T rec_end_request_deal(UINT16_T dest_addr,UINT8_T dest_ep,UINT8_T src_ep,TAL_ZCL_MSG_T frame)
{
    BOOL_T end_req_result=FALSE;
    end_request_pdu_t *req_pdu_buf = (end_request_pdu_t*)tal_malloc(sizeof(end_request_pdu_t));
    end_respond_pdu_t *res_pdu_buf = (end_respond_pdu_t*)tal_malloc(sizeof(end_respond_pdu_t));

    if((req_pdu_buf)&&(res_pdu_buf)){
        tal_system_memset((UINT8_T*)req_pdu_buf,0x00,sizeof(end_request_pdu_t));          
        tal_system_memcpy((UINT8_T*)req_pdu_buf,(UINT8_T*)(frame.payload),sizeof(end_request_pdu_t));
        if(rec_end_request_format_check(frame.payload,frame.length))
        {
            if(default_config.rec_end_req)
            {
                if(SEND_END_SUCCESS == default_config.rec_end_req(dest_addr,*req_pdu_buf))
                {
                    if(res_pdu_buf)
                    {
                        tal_system_memset((UINT8_T*)res_pdu_buf,0x00,sizeof(end_respond_pdu_t));          
                        tal_system_memcpy((UINT8_T*)res_pdu_buf,((UINT8_T*)(frame.payload)+sizeof(UINT8_T)),sizeof(end_respond_pdu_t));
                        transfer_rec_end_respond_send(dest_addr,0x01,get_frist_index_ep(),*res_pdu_buf); 
                        end_req_result=TRUE;                  
                    }
                }
            }  
        } 
    }
    else{
        hal_transfer_debug("tal_malloc fail\r\n");
    }
    if(res_pdu_buf){
        tal_free(res_pdu_buf);
    }
    if(req_pdu_buf){
        tal_free(req_pdu_buf);          
    }
    return end_req_result;
}




static void send_block_respond(UINT16_T src_addr,block_respond_pdu_t pdu_buf)
{
    transfer_block_respond_send(src_addr,0x01,get_frist_index_ep(),pdu_buf);                
}



/*************************************************************************************************
 * 
 * 
 *                                    分包公共部分
 * 
 * 
 * **************************************************************************************************/
/**
 * @description:  在callback中进行接收状态判断
 *                后续的发送帧的调用，而不在接收及发送函数中进行，
 *                内部函数仅进行格式解析并反馈解析结果
 * @param {*frame} ZCL input data
 * @return: none
 */
void multiple_packet_transfer_rec_callback(UINT16_T dest_addr,UINT8_T dest_ep,UINT8_T src_ep,TAL_ZCL_MSG_T frame)
{
    if(src_ep != get_frist_index_ep()){
        return;
    }
    switch(frame.command)
    {
        /*接收数据进程*/
        /*client rec,start notify*/
        case CMD_TRANSFER_START_NOTIFICATION_COMMAND_ID:
            hal_transfer_debug("client rec,start notify \r\n");
            if(rec_start_notification_deal(dest_addr,dest_ep,src_ep,frame)){
                ext_set_quick_time(TRUE,QUICK_POLL_SLICE_TIME);
                set_rec_wait_time(TRUE); 
            }
        break;
        /*client rec,block responds*/
        case CMD_TRANSFER_BLOCK_RESPOND_COMMAND_ID:/*这里需要对接收到的数据进行处理及判断*/
            hal_transfer_debug("client rec,block responds \r\n");
            if(rec_block_respond_deal(dest_addr,dest_ep,src_ep,frame)){
                if(FALSE==tal_sw_timer_is_running(sw_fast_poll_use)){
                    tal_sw_timer_start(sw_fast_poll_use,QUICK_POLL_SLICE_TIME,TAL_TIMER_ONCE);                          
                }
                set_rec_wait_time(TRUE); 
            }
        break;
        /*client rec,end responds*/
        case CMD_TRANSFER_END_RESPOND_COMMAND_ID:/*接收不关心end respond,只需要关系block respond收完了就可以回调应用层了,设备只需要发出end request就可以了*/
            hal_transfer_debug("client rec,end responds \r\n");
#if 0
            if(rec_rec_end_respond_deal(dest_addr,dest_ep,src_ep,frame)){
                hal_transfer_debug("wait timeout end \r\n");                
                set_rec_wait_time(FALSE); 
            }
#endif   
        break;        
        /*发送数据进程*/
        /*Server rec,start notify responds*/
        case CMD_TRANSFER_START_RESPOND_COMMAND_ID:
            hal_transfer_debug("Server rec,start notify responds \r\n");
            if(rec_start_respond_deal(dest_addr,dest_ep,src_ep,frame)){
                if(FALSE==tal_sw_timer_is_running(sw_fast_poll_use)){
                    tal_sw_timer_start(sw_fast_poll_use,QUICK_POLL_SLICE_TIME,TAL_TIMER_ONCE);         
                }
                set_send_wait_time(TRUE);
            }
        break;
        /*Server rec,block request*/
        case CMD_TRANSFER_BLOCK_REQUEST_COMMAND_ID:
            hal_transfer_debug("Server rec,block request \r\n");
            if(rec_block_request_deal(dest_addr,dest_ep,src_ep,frame)){
                if(FALSE==tal_sw_timer_is_running(sw_fast_poll_use)){
                    tal_sw_timer_start(sw_fast_poll_use,QUICK_POLL_SLICE_TIME,TAL_TIMER_ONCE);       
                }
                set_send_wait_time(TRUE);
            }
        break;
        /*Server rec,end request*/
        case CMD_TRANSFER_END_REQUEST_COMMAND_ID:
            hal_transfer_debug("Server rec,end request \r\n");
            if(rec_end_request_deal(dest_addr,dest_ep,src_ep,frame)){
                hal_transfer_debug("wait timeout end \r\n");
                set_send_wait_time(FALSE);
            }
        break;
        default:
        break;
    }
}



BOOL_T multiple_packet_transfer_init(muliple_packet_config_t config,BOOL_T fast_poll,BOOL_T rec_timeout,BOOL_T send_timeout)
{
    BOOL_T result = TRUE;

    /*支持接收分包*/
    if((NULL == config.rec_notify_req)||(NULL == config.rec_block_res)||(NULL == config.rec_end_res)\
     ||(NULL == config.send_block_req)||(NULL == config.send_end_req)){
        return FALSE;
    }
    else{
        default_config.rec_notify_req = config.rec_notify_req;
        default_config.rec_block_res = config.rec_block_res;
        default_config.rec_end_res = config.rec_end_res;
        default_config.send_block_req = config.send_block_req;
        default_config.send_end_req = config.send_end_req;        
    }

    /*支持发送分包*/
    if((NULL == config.rec_notify_res)||(NULL == config.rec_block_req)||(NULL == config.send_block_res)||(NULL == config.rec_end_req)){
        return FALSE;
    }
    else{
        default_config.rec_notify_res = config.rec_notify_res;
        default_config.rec_block_req = config.rec_block_req;
        default_config.send_block_res = config.send_block_res;
        default_config.rec_end_req = config.rec_end_req;       
    }
    /*zigbee发送数据结果回调*/
    if(config.send_result){
        default_config.send_result = config.send_result;
    }
    if(result){
        if(fast_poll){
            if(OPRT_OK==tal_sw_timer_create(multiple_packet_transfer_evt_handle, NULL, &sw_fast_poll_use)){
                result = TRUE;
            }
            else{
                result = FALSE;
            }
        }        
    }
    if(result){
        if(rec_timeout){
            if(OPRT_OK==tal_sw_timer_create(multiple_packet_transfer_evt_handle, NULL, &sw_rec_process_timeout_use)){
                result = TRUE;
            }
            else{
                result = FALSE;
            }
        }        
    }
    if(result){
        if(send_timeout){
            if(OPRT_OK==tal_sw_timer_create(multiple_packet_transfer_evt_handle, NULL, &sw_send_process_timeout_use)){
                result = TRUE;
            }
            else{
                result = FALSE;
            }
        }
    }
    if(!result){
        if(fast_poll){
            if(sw_fast_poll_use){
                tal_sw_timer_delete(sw_fast_poll_use);
            }
        }
        if(rec_timeout){
            if(sw_rec_process_timeout_use){
                tal_sw_timer_delete(sw_rec_process_timeout_use);
            }
        }
        if(send_timeout){
            if(sw_send_process_timeout_use){
                tal_sw_timer_delete(sw_send_process_timeout_use);
            }            
        }
    }
    return result;
}



/**
 * @description: server send,end responds 
 * @param {*data} PDU data,{len}PDU len
 * @return: none
 */
void multiple_packet_transfer_evt_handle(TIMER_ID timer_id, VOID_T *arg)
{
    if(timer_id==sw_fast_poll_use){
        tal_zg_poll_send();
        tal_sw_timer_start(sw_fast_poll_use,g_quick_poll_time,TAL_TIMER_ONCE);            
    }
    else if(timer_id==sw_rec_process_timeout_use){
        hal_transfer_debug("timer_id id: %d\r\n",timer_id);
        if(g_app_try_send_time<(APP_TRY_SEND_TIME_MAX-1)){
            g_app_try_send_time++;
            hal_transfer_debug("block request again %d\r\n",g_app_try_send_time);
            send_block_request_deal();
            tal_sw_timer_start(sw_rec_process_timeout_use,g_app_try_send_buf[g_app_try_send_time],TAL_TIMER_ONCE);
        }
        else{
            hal_transfer_debug("timeout and send end req %d\r\n",g_app_try_send_time);
            send_end_request_deal(END_RES_OTHER_ERROR,TRUE);/*超时后发送end status为失败的状态表示此次同步失败*/ 
            if(FALSE==tal_sw_timer_is_running(sw_send_process_timeout_use)){
                ext_set_quick_time(FALSE,0);                      
            }
                    }
    }
    else if(timer_id==sw_send_process_timeout_use){
        hal_transfer_debug("timer_id id: %d\r\n",timer_id);
        if(FALSE==tal_sw_timer_is_running(sw_rec_process_timeout_use)){
            ext_set_quick_time(FALSE,0);                  
        }
            }
}