/**
* @file tuya_sdk_callback.c
* @brief Common process - adapter the flash api
* @version 0.1
* @date 2021-10-22
*
* @copyright Copyright 2020-2021 Tuya Inc. All Rights Reserved.
*
*/
#include "tuya_tools.h"
#include <stdio.h>
#include "tkl_gpio.h"
#include "tkl_uart.h"
#include "tkl_system.h"
#include "tkl_memory.h"
#include "tkl_network_manager.h"

#include "tal_log.h"
#include "tal_gpio.h"
#include "tal_uart.h"
#include "tal_flash.h"
#include "tal_system.h"
#include "tal_nv_flash.h"
#include "tal_sw_timer.h"
#include "tal_zcl_group.h"
#include "tal_zcl_scene.h"
#include "tal_time_sync.h"
#include "tal_heartbeat.h"
#include "tal_data_send.h"
#include "tal_network_mgr.h"
#include "tal_zcl_identify.h"
#include "tal_data_receive.h"
#include "tal_attribute_rw.h"
#include "tal_firmware_cfg.h"
#include "tal_reset_factory.h"
#include "tal_zll_commissioning.h"
#include "tal_endpoint_register.h"
#include "tal_memory.h"

#include "app_common.h"
#include "app_config.h"
#include "tdd_file_transfer.h"

#include "tlPrintf.h"

TIMER_ID etimer_key_scan;
TIMER_ID etimer_led_blink_event_id;
TIMER_ID etimer_join_start_delay;
TIMER_ID etimer_mf_blink;
TIMER_ID etimer_report_ir_version;



#define TUYA_SKD_DBG    1
#if defined (TUYA_SKD_DBG)
#include "tkl_uart.h"
#include "tlprintf.h"
#define sdk_callback_debug(...)   tkl_dbg_printf(__VA_ARGS__)
#else
#define sdk_callback_debug(...)
#endif

#ifdef ENABLE_TAL_LOG
const char *g_net_st_str_test[] = {
    "TAL_ZG_NWK_IDLE",            ///< inner using
    "TAL_ZG_NWK_POWER_ON_LEAVE",  ///< power on and device is not joined network
    "TAL_ZG_NWK_POWER_ON_ONLINE", ///< power on and device is already joined network
    "TAL_ZG_NWK_JOIN_START",      ///< start joining network
    "TAL_ZG_NWK_JOIN_TIMEOUT",    ///< network joining timeout
    "TAL_ZG_NWK_JOIN_OK",         ///< network joined success
    "TAL_ZG_NWK_LOST",            ///< network lost, lost parent
    "TAL_ZG_NWK_REJOIN_OK",       ///< network rejoin ok
    "TAL_ZG_NWK_REMOTE_LEAVE",    ///< remove device by remote device
    "TAL_ZG_NWK_LOCAL_LEAVE",     ///< remove device by local
    "TAL_ZG_NWK_MF_TEST_LEAVE",   ///< remove device by PC test tools
    "TAL_ZG_NWK_ZLL_JOINED",      ///< network joined zll network
    "TAL_ZG_NWK_ZLL_LEAVE",       ///< remove device Zll Reset To Factory New
};
#endif

#ifndef GET_ARRAY_LEN
#define GET_ARRAY_LEN(x) (sizeof(x) / sizeof(x[0]))
#endif

#define IDENTIFY_ATTR_LIST                                                                                                     \
    {0x0000, ATTR_INT16U_ATTRIBUTE_TYPE, 2, (ATTR_MASK_TOKEN_FAST),0, {(UINT8_T *)0x0005}},   /* 12 / Identify / identify time*/ \
        {0xFFFD, ATTR_INT16U_ATTRIBUTE_TYPE, 2, (ATTR_MASK_READABLE), 0,{(UINT8_T *)0x0001}}, /* 13 / Identify / cluster revision*/

#define IDENTIFY_CLIENT_ATTR_LIST \
    {0xFFFD, ATTR_INT16U_ATTRIBUTE_TYPE, 2, (ATTR_MASK_CLIENT),0, ((UINT8_T *)0x0001)}, /* 13 / Identify / cluster revision*/

#define ON_OFF_LIGHT_ATTR_LIST                                                                                      \
    {0x0000, ATTR_BOOLEAN_ATTRIBUTE_TYPE, 1, (ATTR_MASK_READABLE),0, (UINT8_T *)0x00},      /* 20 / On/off / on/off*/ \
        {0xFFFD, ATTR_INT16U_ATTRIBUTE_TYPE, 2, (ATTR_MASK_READABLE), 0,(UINT8_T *)0x0002}, /* 24 / On/off / cluster revision*/

#define GROUP_ATTR_LIST                                                                                                   \
    {0x0000, ATTR_BITMAP8_ATTRIBUTE_TYPE, 1, (ATTR_MASK_READABLE), 0,(UINT8_T *)0x00},      /* 12 / Groups / name support*/ \
        {0xFFFD, ATTR_INT16U_ATTRIBUTE_TYPE, 2, (ATTR_MASK_READABLE), 0,(UINT8_T *)0x0002}, /* 13 / Groups / cluster revision*/

#define SCENE_ATTR_LIST                                                                                                    \
    {0x0000, ATTR_INT8U_ATTRIBUTE_TYPE, 1, (ATTR_MASK_READABLE),0, (UINT8_T *)0x00},        /* 15 / Scenes / scene count*/   \
        {0x0001, ATTR_INT8U_ATTRIBUTE_TYPE, 1, (ATTR_MASK_READABLE),0, (UINT8_T *)0x00},    /* 16 / Scenes / current scene*/ \
        {0x0002, ATTR_INT16U_ATTRIBUTE_TYPE, 2, (ATTR_MASK_READABLE), 0,(UINT8_T *)0x0000}, /* 16 / Scenes / current group*/ \
        {0x0003, ATTR_BOOLEAN_ATTRIBUTE_TYPE, 1, (ATTR_MASK_READABLE),0, (UINT8_T *)0x00},  /* 17 / Scenes / scene valid*/   \
        {0x0004, ATTR_BITMAP8_ATTRIBUTE_TYPE, 1, (ATTR_MASK_READABLE), 0,(UINT8_T *)0x00},  /* 18 / Scenes / name support*/  \
        {0xFFFD, ATTR_INT16U_ATTRIBUTE_TYPE, 2, (ATTR_MASK_READABLE), 0,(UINT8_T *)0x0002}, /* 19 / Scenes / cluster revision*/

#define LEVEL_CONTROL_ATTR_LIST                                                                                                             \
    {0x0000, ATTR_INT8U_ATTRIBUTE_TYPE, 1, (ATTR_MASK_TOKEN_FAST), 0,(UINT8_T *)0x01},        /* 20 / Level Control / Current Level*/         \
        {0xF000, ATTR_INT16U_ATTRIBUTE_TYPE, 2, (ATTR_MASK_TOKEN_FAST),0, (UINT8_T *)0x0001}, /* 20 / Private Level Control / Current Level*/ \
        {0xFFFD, ATTR_INT16U_ATTRIBUTE_TYPE, 2, (ATTR_MASK_READABLE),0, (UINT8_T *)0x0001},

#define COLOR_CONTROL_ATTR_LIST                                                                                                      \
    {0xF000, ATTR_INT8U_ATTRIBUTE_TYPE, 1, (ATTR_MASK_READABLE),0, (UINT8_T *)0x00},      /* 20 / Private Color Control / color mode*/ \
        {0xF102, ATTR_ARRAY_ATTRIBUTE_TYPE, 13, (ATTR_MASK_READABLE),0, (UINT8_T *)0x00}, /* app  data*/                               \
        {0xFFFD, ATTR_INT16U_ATTRIBUTE_TYPE, 2, (ATTR_MASK_READABLE),0, (UINT8_T *)0x0001},


#define  ZLL_ATTR_LIST  \
    { 0xFFFD, ATTR_INT16U_ATTRIBUTE_TYPE, 2, 0, 0, (UINT8_T*)0x0002 },

#define MULTIPLE_PACKET_TRANSFER_ATTR_LIST \
    { 0xFFFD, ATTR_INT16U_ATTRIBUTE_TYPE, 2, (ATTR_MASK_SINGLETON), 0, (UINT8_T*)0x0001  }, /* tuya large file transfer / cluster revision*/

#define IR_CONTROL_ATTR_LIST \
    { 0x0000, ATTR_INT16U_ATTRIBUTE_TYPE, 2, (ATTR_MASK_SINGLETON), 0,  (UINT8_T*)0x0003  }, /*  cluster revision*/\
 

const TAL_ATTR_T g_multiple_packet_transfer_attr_list[] = {
    MULTIPLE_PACKET_TRANSFER_ATTR_LIST
    };

const TAL_ATTR_T g_ir_control_attr_list[] = {
    IR_CONTROL_ATTR_LIST
};    


// application attribute
const TAL_ATTR_T identify_attr_list[] = {
    IDENTIFY_ATTR_LIST};

const TAL_ATTR_T onoff_attr_list[] = {
    ON_OFF_LIGHT_ATTR_LIST};

const TAL_ATTR_T group_attr_list[] = {
    GROUP_ATTR_LIST};

const TAL_ATTR_T scene_attr_list[] = {
    SCENE_ATTR_LIST};

const TAL_ATTR_T level_attr_list[] = {
    LEVEL_CONTROL_ATTR_LIST};

const TAL_ATTR_T color_attr_list[] = {
    COLOR_CONTROL_ATTR_LIST};

const TAL_ATTR_T zll_attr_list[] = {
    ZLL_ATTR_LIST};


#define DEF_CLUSTER_ELECTRICAL_MEASUREMENT_CLUSTER_ID(a) \
    {CLUSTER_ELECTRICAL_MEASUREMENT_CLUSTER_ID, (TAL_ATTR_T *)&((a)[0]), GET_ARRAY_LEN((a))},

#define DEF_CLUSTER_IDENTIFY_CLUSTER_ID(a) \
    {CLUSTER_IDENTIFY_CLUSTER_ID, (TAL_ATTR_T *)&((a)[0]), GET_ARRAY_LEN((a))},

#define DEF_CLUSTER_ON_OFF_CLUSTER_ID(a) \
    {CLUSTER_ON_OFF_CLUSTER_ID, (TAL_ATTR_T *)&((a)[0]), GET_ARRAY_LEN((a))},

#define DEF_CLUSTER_GROUPS_CLUSTER_ID(a) \
    {CLUSTER_GROUPS_CLUSTER_ID, (TAL_ATTR_T *)&((a)[0]), GET_ARRAY_LEN((a))},

#define DEF_CLUSTER_SCENES_CLUSTER_ID(a) \
    {CLUSTER_SCENES_CLUSTER_ID, (TAL_ATTR_T *)&((a)[0]), GET_ARRAY_LEN((a))},

#define DEF_CLUSTER_LEVEL_CONTROL_CLUSTER_ID(a) \
    {CLUSTER_LEVEL_CONTROL_CLUSTER_ID, (TAL_ATTR_T *)&((a)[0]), GET_ARRAY_LEN((a))},

#define DEF_CLUSTER_COLOR_CONTROL_CLUSTER_ID(a) \
    {CLUSTER_COLOR_CONTROL_CLUSTER_ID, (TAL_ATTR_T *)&((a)[0]), GET_ARRAY_LEN((a))},

#define DEF_CLUSTER_ZLL_COMMISSIONING_CLUSTER_ID(a) \
    { CLUSTER_ZLL_COMMISSIONING_CLUSTER_ID, (TAL_ATTR_T *)&((a)[0]), GET_ARRAY_LEN((a)) },

#define DEF_CLUSTER_MULTIPLE_PACKET_TRANSFER_CLUSTER_ID(a) \
    { CLUSTER_MULTIPLE_PACKET_TRANSFER_CLUSTER_ID , (TAL_ATTR_T *)&((a)[0]), GET_ARRAY_LEN((a)) },

#define DEF_CLUSTER_IR_CONTROL_CLUSTER_ID(a) \
    { CLUSTER_TUYA_IR_CONTROL_CLUSTER_ID , (TAL_ATTR_T *)&((a)[0]), GET_ARRAY_LEN((a)) },


// server cluster
const TAL_CLUSTER_T app_server_cluster_list[] = {
    DEF_CLUSTER_IDENTIFY_CLUSTER_ID(identify_attr_list)
    DEF_CLUSTER_GROUPS_CLUSTER_ID(group_attr_list)
    DEF_CLUSTER_SCENES_CLUSTER_ID(scene_attr_list)
    DEF_CLUSTER_MULTIPLE_PACKET_TRANSFER_CLUSTER_ID(g_multiple_packet_transfer_attr_list)
    DEF_CLUSTER_IR_CONTROL_CLUSTER_ID(g_ir_control_attr_list)};
                        

#define SERVER_CLUSTER_NUM GET_ARRAY_LEN(app_server_cluster_list)

// server cluster
const TAL_CLUSTER_T app_client_cluster_list[] = {
    DEF_CLUSTER_MULTIPLE_PACKET_TRANSFER_CLUSTER_ID(g_multiple_packet_transfer_attr_list)};
                        

#define CLIENT_CLUSTER_NUM GET_ARRAY_LEN(app_client_cluster_list)

#define ZG_DEVICE_ID_WIRELESS_CONTROL   0xF000

TAL_ENDPOINT_T dev_endpoint_desc[] = {
    {1, ZHA_PROFILE_ID, ZG_DEVICE_ID_WIRELESS_CONTROL, SERVER_CLUSTER_NUM, (TAL_CLUSTER_T *)&app_server_cluster_list[0], CLIENT_CLUSTER_NUM, (TAL_CLUSTER_T *)&app_client_cluster_list[0]},
};


VOID_T app_ir_report_info_cb(TIMER_ID timer_id, VOID_T *arg);


/**
 * @note join start delay timer hander, join now
 * @param[in]  none
 * @param[out] none
 * @return none
 */
STATIC VOID_T __app_network_join_start_delay_cb(TIMER_ID timer_id, VOID_T *arg)
{
    tal_zg_join_start(ZIGBEE_JOIN_MAX_TIMEOUT);
}

/**
 * @brief zigbee node init
 * 
 * @return VOID_T 
 */
STATIC VOID_T __app_router_node_init(VOID_T)
{
    TAL_ZG_NODE_CFG_T node_config = {
        .node_type = ZG_ROUTER,
        .tx_power = 11,
        .scan_interval = 0,
        .scan_duration = ZG_SCAN_DURATION_3,
    };
    tal_zg_node_config(&node_config);
}
/**
 * @brief Generally used for peripheral initialization
 * 
 * @return OPERATE_RET 
 */
OPERATE_RET tuya_init_first(VOID_T)
{
    TAL_UART_CFG_T uart_cfg = {
        .rx_buffer_size = 256,
        .open_mode = 0,
        {
            .baudrate = 115200,
            .parity = TUYA_UART_PARITY_TYPE_NONE,
            .databits = TUYA_UART_DATA_LEN_8BIT,
            .stopbits = TUYA_UART_STOP_LEN_1BIT,
            .flowctrl = TUYA_UART_FLOWCTRL_NONE,
        }
    };
    tal_uart_init(USER_UART0, &uart_cfg);

    app_button_init();

    app_status_led_init();

#if (ENABLE_TAL_LOG == 1)
    tal_log_create_manage_and_init(TAL_LOG_LEVEL_DEBUG, 128, Tl_printf);
#endif

    TAL_PR_DEBUG("/*********first init*********/");
    return OPRT_OK;
}
/**
 * @brief Generally used for register zigbee device
 * 
 * @return OPERATE_RET 
 */
OPERATE_RET tuya_init_second(VOID_T)
{
    //initialize firmware infomation
    TAL_PR_DEBUG("Application version:%d", APP_FIRMWARE_VER);

    //creat software timer

    tal_sw_timer_create(app_button_scan_cb, NULL, &etimer_key_scan);
    tal_sw_timer_create(__app_network_join_start_delay_cb, NULL, &etimer_join_start_delay);
    tal_sw_timer_create(app_ir_report_info_cb, NULL, &etimer_report_ir_version);

    //register zigbee endpoint
    tal_zg_endpoint_register(dev_endpoint_desc, GET_ARRAY_LEN(dev_endpoint_desc));

    //zigbee node configuration
    __app_router_node_init();

    //zigbee joining network configuration
    TAL_ZG_JOIN_CFG_T join_config = {
        .auto_join_power_on_flag = TRUE,
        .auto_join_remote_leave_flag = TRUE,
        .join_timeout = ZIGBEE_JOIN_MAX_TIMEOUT,
    };
    tal_zg_join_config(&join_config);

    tal_time_sync_period_set(60 * 1000);
    
    TAL_PR_DEBUG("/*********1.0.0 second init*********/");
    return OPRT_OK;
}
/**
 * @brief Generally used for initialization before manufacturing test
 * 
 * @return OPERATE_RET 
 */
OPERATE_RET tuya_init_third(VOID_T)
{
    TAL_PR_DEBUG("/*********third init*********/");

    return OPRT_OK;
}
/**
 * @brief Generally used for initialization after manufacturing test
 * 
 * @return OPERATE_RET 
 */
OPERATE_RET tuya_init_last(VOID_T)
{
    UINT8_T version;
    
    tal_uart_deinit(USER_UART0);
    TAL_UART_CFG_T uart_cfg = {
        .rx_buffer_size = 256,
        .open_mode = 0,
        {
            .baudrate = 115200,
            .parity = TUYA_UART_PARITY_TYPE_NONE,
            .databits = TUYA_UART_DATA_LEN_8BIT,
            .stopbits = TUYA_UART_STOP_LEN_1BIT,
            .flowctrl = TUYA_UART_FLOWCTRL_NONE,
        }
    };
    tal_uart_init(USER_UART0, &uart_cfg);

    tdd_file_transfer_init(TRUE,TRUE,FALSE,TRUE,TRUE); 

    tal_zg_read_attribute(TUYA_PRIMARY_ENDPOINT,
                          CLUSTER_BASIC_CLUSTER_ID,
                          ATTR_APPLICATION_VERSION_ATTRIBUTE_ID,
                          (VOID_T *)&version,
                          sizeof(version));

    TAL_PR_DEBUG("/*********last init %d*********/", version);
    return OPRT_OK;
}
/**
 * @brief user-defined callback interface in main loop.do not block!!!
 * 
 * @return OPERATE_RET 
 */
OPERATE_RET tuya_main_loop(VOID_T)
{
    return OPRT_OK;
}
/**
 * @brief heartbeat report callback
 * NOTE: you can rewrite this API in application layer
 * 
 * @param[in]   status: heartbeat report status
 *
 * @return none
 */
VOID_T tal_heartbeat_report_callback(TAL_HEARTBEAT_TYPE_E type)
{
    TAL_PR_DEBUG("heartbeat: %d", type);
}
/**
 * @brief general message receive callback
 * 
 * @param msg 
 * @return TAL_MSG_RET_E 
 */
TAL_MSG_RET_E tal_zcl_general_msg_recv_callback(TAL_ZCL_MSG_T *msg)
{
    //TAL_PR_DEBUG("app gen msg cb: cluster 0x%02x, cmd %d", msg->cluster, msg->command);

    return ZCL_MSG_RET_SUCCESS;
}
/**
 * @brief specific message receive callback
 * 
 * @param msg 
 * @return TAL_MSG_RET_E 
 */
TAL_MSG_RET_E tal_zcl_specific_msg_recv_callback(TAL_ZCL_MSG_T *msg)
{

    TAL_PR_DEBUG("app spec msg cb: cluster 0x%x, cmd 0x%x", msg->cluster, msg->command);

    ZIGBEE_CMD_T app_cmd_type = ZIGBEE_CMD_SINGLE;
    if (msg->mode == ZG_UNICAST_MODE)
    {
        app_cmd_type = ZIGBEE_CMD_SINGLE;
        TAL_PR_DEBUG("receive single message");
    }
    else
    {
        app_cmd_type = ZIGBEE_CMD_GROUP;
        TAL_PR_DEBUG("receive group message");
    }

    switch (msg->cluster)
    {
        case CLUSTER_MULTIPLE_PACKET_TRANSFER_CLUSTER_ID:
        {
            switch(msg->command)
            {
                ///< Client to server
                case CMD_TRANSFER_START_RESPOND_COMMAND_ID:
                case CMD_TRANSFER_BLOCK_REQUEST_COMMAND_ID:
                case CMD_TRANSFER_END_REQUEST_COMMAND_ID:
                ///< Server to client
                case CMD_TRANSFER_START_NOTIFICATION_COMMAND_ID:  
                case CMD_TRANSFER_BLOCK_RESPOND_COMMAND_ID:
                case CMD_TRANSFER_END_RESPOND_COMMAND_ID:
                {
                    multiple_packet_transfer_rec_callback(msg->src_addr,0xff,msg->src_ep,*msg);
                    break;  
                }                 
                default:
                {
                    break;
                }
            }
            break;
        }
        case CLUSTER_TUYA_IR_CONTROL_CLUSTER_ID:
        {
            UINT32_T len = msg->length;
            UINT8_T *data = msg->payload;

            /*分包len>0xff的数据时，len为0xff*/
            if(0xFF==len){
                len = *(UINT32_T*)(msg->payload);
                data = (UINT8_T*)msg->payload+sizeof(UINT32_T);
            }                
            TAL_PR_DEBUG("0x%x\r\n",msg->command);

            for(UINT32_T i=0;i<len;i++){
                sdk_callback_debug("%c",data[i]);
            }
            sdk_callback_debug("\r\n");

            switch(msg->command)
            {
                ///< Client to server
                case CMD_IR_STUDY_STATUS_COMMAND_ID:
                {
                    break;  
                }                
                case CMD_IR_KEYID_CONTROL_COMMAND_ID:
                {
                    break;  
                }
                case CMD_IR_KEYCODE_SEND_COMMAND_ID:
                {
                    break;  
                }
                case CMD_IR_DELETE_KEYCODE_COMMAND_ID:
                {
                    break;  
                }
                case CMD_IR_KEYCODE_IMMEDIATE_DOWNLOAD_COMMAND_ID:
                {
                    break;  
                }  
                case CMD_IR_KEYCODE_DELAY_DOWNLOAD_COMMAND_ID:
                {
                    break;  
                }  
                case CMD_IR_KEYCODE_REPLACE_COMMAND_ID:
                {
                    break;
                }              
                default:
                {
                    break;
                }
            }
            break;
        }
        default:
            break;
    }

    return ZCL_MSG_RET_SUCCESS;
}
/**
 * @brief pre-save scene callback
 * 
 * @param ep_id 
 * @param scene_id 
 * @param group_id 
 * @return VOID_T 
 */
VOID_T tal_zg_scene_pre_save_callback(UINT8_T ep_id, UINT8_T scene_id, UINT16_T group_id)
{
    TAL_PR_DEBUG("scene pre save: ep %d, sce %d, gp 0x%02x", ep_id, scene_id, group_id);
}
/**
 * @brief save scene callback(add scene and store scene)
 * 
 * @param ep_id 
 * @param scene_id 
 * @param group_id 
 * @param data 
 * @return VOID_T 
 */
VOID_T tal_zg_scene_save_callback(UINT8_T ep_id, UINT8_T scene_id, UINT16_T group_id, TAL_SCENE_DATA_T *data)
{
    TAL_PR_DEBUG("scene save: ep %d, sce %d, gp 0x%x, type %d", ep_id, scene_id, group_id, data->type);
}
/**
 * @brief recall scene callback
 *
 * @param[in]   ep_id:     endpoint id
 * @param[in]   scene_id:  scene id
 * @param[in]   group_id:  group id
 * @param[in]   time100ms: scene transition time(bat:100ms)
 * @param[in]   data:      point to scene data
 * @return  VOID_T
 */
VOID_T tal_zg_scene_recall_callback(UINT8_T ep_id, UINT8_T scene_id, UINT16_T group_id, UINT_T time100ms, TAL_SCENE_DATA_T *data)
{
    TAL_PR_DEBUG("scene recall: ep %d, sce %d, gp 0x%x, type %d", ep_id, scene_id, group_id, data->type);
}
/**
 * @brief add group callback
 *
 * @param[in]   ep_id:    endpoint id
 * @param[in]   group_id: group id
 * @return  VOID_T
 */
VOID_T tal_zg_add_group_callback(UINT8_T ep_id, UINT16_T group_id)
{
    TAL_PR_DEBUG("ep id: %d,group id: %d", ep_id, group_id);
}
/**
 * @brief view group callback
 *
 * @param[in]   ep_id:    endpoint id
 * @param[in]   group_id: group id
 * @return  VOID_T
 */
VOID_T tal_zg_view_group_callback(UINT8_T ep_id, UINT16_T group_id)
{
    TAL_PR_DEBUG("ep id: %d,group id: %d", ep_id, group_id);
}
/**
 * @brief remove group callback
 *
 * @param[in]   ep_id:    endpoint id
 * @param[in]   group_id: group id
 * @return  VOID_T
 */
VOID_T tal_zg_remove_group_callback(UINT8_T ep_id, UINT16_T group_id)
{
    TAL_PR_DEBUG("ep id: %d,group id: %d", ep_id, group_id);
}
/**
 * @brief zigbee network network change callback(user can rewrite this API)
 * 
 * @param[in]   status: network status
 * @return VOID_T
 */
VOID_T tal_zg_nwk_status_changed_callback(TAL_ZG_NWK_STATUS_E status)
{
    switch (status)
    {
    case TAL_ZG_NWK_POWER_ON_LEAVE:
    {
        TAL_PR_DEBUG("power_on_leave---");
        break;
    }
    case TAL_ZG_NWK_POWER_ON_ONLINE:
    {
        tal_sw_timer_start(etimer_report_ir_version, 500, TAL_TIMER_ONCE);
        TAL_PR_DEBUG("power_on_online---");
        break;
    }
    case TAL_ZG_NWK_JOIN_START:
    {
        TAL_PR_DEBUG("nwk_join_start---");
        //light blink when steering
        status_led_start_blink(TRUE);
        break;
    }
    case TAL_ZG_NWK_JOIN_OK:
    {
        tal_sw_timer_start(etimer_report_ir_version, 500, TAL_TIMER_ONCE);
        TAL_PR_DEBUG("nwk_join_ok---");
        status_led_stop_blink(FALSE);
        break;
    }
    case TAL_ZG_NWK_REJOIN_OK:
    {
        TAL_PR_DEBUG("nwk_rejoin_ok---");
        break;
    }
    case TAL_ZG_NWK_JOIN_TIMEOUT:
    {
        TAL_PR_DEBUG("nwk_join_timeout---");
        status_led_stop_blink(FALSE);
        break;
    }
    case TAL_ZG_NWK_LOST:
    {
        TAL_PR_DEBUG("nwk_lost---");
        break;
    }
    case TAL_ZG_NWK_REMOTE_LEAVE:
    {
        TAL_PR_DEBUG("nwk_remote_leave---");
        status_led_stop_blink(FALSE);
        break;
    }
    case TAL_ZG_NWK_LOCAL_LEAVE:
    {
        TAL_PR_DEBUG("nwk_local_leave---");
        break;
    }
    case TAL_ZG_NWK_MF_TEST_LEAVE:
    {
        TAL_PR_DEBUG("nwk_mf_test_leave---");
        break;
    }
    default:
    {
        break;
    }
    }
}
VOID_T tal_beacon_mf_test_callback(VOID_T)
{
    TAL_PR_DEBUG("enter beacon test\n");
}

/**
 * @brief reset factory default callback
 *
 * @param[in]   type: reset factory type
 * @return  VOID_T
 */
VOID_T tal_zg_reset_factory_default_callback(TAL_RESET_TYPE_T type)
{
    TAL_PR_DEBUG("receive reset to factory default cmd %d", type);
}

VOID_T tal_zll_target_commissioning_complete_callback(TAL_ZLL_TARGET_STATUS_E status)
{
    TAL_PR_DEBUG("zll target status %d",status);
}



VOID_T tal_time_sync_complete_callback(BOOL_T status, UINT_T time_sec)
{
 }


#include "tkl_platform_types.h"

BOOL_T app_print_get_cfg(UINT8_T *print_type, UINT8_T *disable_irq, GPIO_PORT_PIN_T *gpio)
{
    gpio->port = PORT_C;
    gpio->pin = PIN_4;
    *print_type = 0;
    *disable_irq = 1;
#ifdef UART_PRINTF_MODE
    return 1;
#else
    return 0;
#endif
}

#define IR_VIRID_INFO_ATTR_ID       0x0001/*子设备入网后上报剩余可以匹配的虚拟子设备数量，及在线的虚拟子设备vir_id*/
#define IR_DEVICE_NUM_MAX           10/*最多10个设备*/

static void dev_vir_info_report(void)
{
    TAL_ZG_SEND_DATA_T ret_data;
    TAL_ZG_NWK_STATUS_E net_states = tal_zg_nwk_status_get();
    char out[64] = "\0";

    if((TAL_ZG_NWK_POWER_ON_ONLINE == net_states) || (TAL_ZG_NWK_JOIN_OK == net_states) || (TAL_ZG_NWK_REJOIN_OK == net_states)) 
    {
        tal_system_memset(out, 0, sizeof(out));
        tal_system_memset(&ret_data, 0, sizeof(ret_data));
        TAL_ATTR_VALUE_T *ret_attr = ret_data.data.zg.attr;
        ret_data.frame_type = ZG_ZCL_FRAME_TYPE_GLOBAL;
        ret_data.command_id = CMD_REPORT_ATTRIBUTES_COMMAND_ID;
        ret_data.direction = ZG_ZCL_DATA_SERVER_TO_CLIENT;
        ret_data.qos = QOS_1;
        ret_data.addr.type.dev.src_ep = 1;
        ret_data.addr.type.dev.dst_ep = 1;
        ret_data.addr.type.dev.cluster_id = CLUSTER_TUYA_IR_CONTROL_CLUSTER_ID;
        ret_attr[0].attr_id = IR_VIRID_INFO_ATTR_ID;
        ret_attr[0].type = ATTR_CHAR_STRING_ATTRIBUTE_TYPE;
        snprintf(out, sizeof(out),"{\"num\":%d}",IR_DEVICE_NUM_MAX);
        TAL_PR_DEBUG("report string %s\r\n",out);
        ret_attr[0].value[0] = strlen(out);    
        tal_system_memcpy(&ret_attr[0].value[1],out,strlen(out));
        ret_attr[0].size = (strlen(out)+1);
        ret_data.data.zg.attr_sum = 1;        
        tal_zg_send_data(&ret_data, NULL, 3000);     
    }
}


VOID_T app_ir_report_info_cb(TIMER_ID timer_id, VOID_T *arg)
{
    dev_vir_info_report();
}


static bool_t notify_flag = false;
static UINT8_T *start_buf = NULL;
bool_t app_start_notify_callback(UINT16_T addr,start_notification_pdu_t pdu,UINT8_T output_len,UINT8_T **input_buf,UINT16_T *malloc_len)
{
    bool_t result = FALSE;

    *malloc_len = 2 * 1024 + output_len;
    if(start_buf == NULL){
        start_buf = (UINT8_T*)tal_malloc(*malloc_len);
    }    
    if(1/*(pdu.cluster_id == CLUSTER_TUYA_IR_CONTROL_CLUSTER_ID) && (pdu.cmd == CMD_IR_KEYCODE_SEND_COMMAND_ID)*/)
    {
        if(start_buf != NULL)
        {
            if(notify_flag == false)
            {
                notify_flag = true;
                *input_buf = start_buf;
                result = true;
            }
        }
    }
    return result;
}

bool_t file_rec_transfer_end_callback(UINT16_T addr,UINT32_T file_id,bool_t result,UINT8_T *buf,UINT16_T len)
{
    TAL_PR_DEBUG("file_id %x recv result:%x\r\n",file_id,result);
    notify_flag = false;
    if(buf){
        tal_free(buf);
        buf = NULL;
    }
    return true;
}

bool_t file_send_transfer_end_callback(UINT16_T addr,UINT32_T file_id,bool_t result,UINT8_T *buf,UINT16_T len)
{
    TAL_PR_DEBUG("file_id %x send result:%x\r\n",file_id,result);
    if(buf){
        tal_free(buf);
        buf = NULL;
    }
    return true;
}